package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.api.commons.security.bean.UserDataRule;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.admin.base.utils.Check;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.ExamScoreRepo;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.admin.dao.enums.MarkingType;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.ExamStudentEffectiveScoreInfo;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.UserDataRules;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.examrecord.ExamRecordQuery;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 考试记录接口
 *
 * @author: QMTH
 * @since: 2018/8/16
 */
@Service
public class ExamRecordServiceImpl implements ExamRecordService {

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public List<ExamRecordDataEntity> getExamRecordListByExamStudentId(Long examStudentId) {
        Check.isNull(examStudentId, "考生ID不能为空！");
        return examRecordDataRepo.findByExamStudentId(examStudentId);
    }

    @Override
    public Long existsWarnExamRecordDetail(UserDataRules uds, ExamRecordQuery query) {
        Check.isNull(query, "请求参数不能为空！");
        Check.isNull(query.getExamId(), "请先选择考试！");

        UserDataRule courseRule = uds.getCourseRule();
        UserDataRule orgRule = uds.getOrgRule();
        if (courseRule.assertEmptyQueryResult() || orgRule.assertEmptyQueryResult()) {
            return 0L;
        }

        StringBuilder sql = new StringBuilder();
        sql.append(" select count(rd.id) from ec_oe_exam_record_data rd");
        sql.append(" where rd.exam_id = ").append(query.getExamId());
        sql.append(" and rd.exam_record_status in ('EXAM_END','EXAM_OVERDUE')");
        sql.append(" and rd.is_warn = 1 and rd.is_audit = 0");

        if (query.getExamStageId() != null) {
            sql.append(" and rd.exam_stage_id = " + query.getExamStageId());
        }
        if (StringUtils.isNotBlank(query.getStudentCode())) {
            sql.append(" and rd.student_code like '" + query.getStudentCode() + "%'");
        }
        if (StringUtils.isNotBlank(query.getIdentityNumber())) {
            sql.append(" and rd.identity_number like '" + query.getIdentityNumber() + "%'");
        }
        if (StringUtils.isNotBlank(query.getStudentName())) {
            sql.append(" and rd.student_name like '" + query.getStudentName() + "%'");
        }
        if (StringUtils.isNotBlank(query.getCourseLevel())) {
            sql.append(" and rd.course_level = '" + query.getCourseLevel() + "'");
        }

        if (courseRule.assertNeedQueryRefIds()) {
            // 限定课程数据权限范围
            if (query.getCourseId() != null) {
                if (courseRule.getRefIds().contains(query.getCourseId())) {
                    sql.append(" and rd.course_id = ").append(query.getCourseId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.course_id = -1");
                }
            } else {
                sql.append(" and rd.course_id in (").append(StringUtils.join(courseRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getCourseId() != null) {
                sql.append(" and rd.course_id = ").append(query.getCourseId());
            }
        }

        if (orgRule.assertNeedQueryRefIds()) {
            // 限定机构数据权限范围
            if (query.getOrgId() != null) {
                if (orgRule.getRefIds().contains(query.getOrgId())) {
                    sql.append(" and rd.org_id = ").append(query.getOrgId());
                } else {
                    // 不在数据权限范围内，无效查询
                    sql.append(" and rd.org_id = -1");
                }
            } else {
                sql.append(" and rd.org_id in (").append(StringUtils.join(orgRule.getRefIds(), ",")).append(")");
            }
        } else {
            // 未限定数据权限
            if (query.getOrgId() != null) {
                sql.append(" and rd.org_id = ").append(query.getOrgId());
            }
        }

        return jdbcTemplate.queryForObject(sql.toString(), Long.class);
    }

    @Override
    public List<ExamStudentEffectiveScoreInfo> getExamStudentEffectiveScoreList(String markingType, List<Long> examStudentIdList) {
        List<ExamStudentEffectiveScoreInfo> resultList = new ArrayList<>();
        //考生所有的考试记录
        List<ExamRecordDataEntity> allExamRecordDataList = examRecordDataRepo.findEffectiveDataByExamStudentIdList(examStudentIdList);
        if (allExamRecordDataList == null || allExamRecordDataList.isEmpty()) {
            return null;
        }
        List<Long> examRecordDataIdList = allExamRecordDataList.stream().map(p -> p.getId()).collect(Collectors.toList());
        //当前考试记录的所有成绩
        List<ExamScoreEntity> allExamScoreList = examScoreRepo.findByExamRecordDataIdIn(examRecordDataIdList);
        if (allExamScoreList == null || allExamScoreList.isEmpty()) {
            return null;
        }
        Map<Long, List<ExamRecordDataEntity>> groupedByExamStudentExamRecordMap = allExamRecordDataList.stream().collect(Collectors.groupingBy(ExamRecordDataEntity::getExamStudentId));
        for (Long examStudentId : groupedByExamStudentExamRecordMap.keySet()) {
            //当前考生的所有考试记录集合
            List<ExamRecordDataEntity> curExamRecordDataList = groupedByExamStudentExamRecordMap.get(examStudentId);
            //当前考试记录id集合
            List<Long> curExamRecordDataIdList = curExamRecordDataList.stream().map(p -> p.getId()).collect(Collectors.toList());

            //当前考生的所有考试成绩集合
            List<ExamScoreEntity> curExamScoreList = allExamScoreList.stream().
                    filter(p -> curExamRecordDataIdList.contains(p.getExamRecordDataId())).collect(Collectors.toList());

            //最终生效的成绩
            ExamScoreEntity effectiveScore = getStudentEffectiveScore(curExamScoreList, markingType);
            //最终生效的考试记录
            ExamRecordDataEntity effectiveExamRecordData = curExamRecordDataList.stream().
                    filter(p -> p.getId().equals(effectiveScore.getExamRecordDataId())).collect(Collectors.toList()).get(0);

            ExamStudentEffectiveScoreInfo info = new ExamStudentEffectiveScoreInfo();
            info.setExamStudentId(examStudentId);
            info.setRootOrgId(effectiveExamRecordData.getRootOrgId());
            info.setOrgId(effectiveExamRecordData.getOrgId());
            info.setBasePaperId(effectiveExamRecordData.getBasePaperId());
            info.setCourseId(effectiveExamRecordData.getCourseId());
            info.setScore(effectiveScore.getTotalScore());
            info.setExamRecordDataId(effectiveExamRecordData.getId());
            resultList.add(info);
        }

        return resultList;
    }

    /**
     * @Description 获取考生一次考试的最终有效成绩
     * @Author lideyin
     * @Date 2019/7/18 17:49
     * @Version 1.0
     */
    private ExamScoreEntity getStudentEffectiveScore(List<ExamScoreEntity> curExamScoreList, String markingType) {
        if (curExamScoreList == null || curExamScoreList.isEmpty()) {
            throw new StatusException("201101", "考试记录不为空，考试分数数据为空");
        }
        ExamScoreEntity examScoreEntity;
        //阅卷方式为全部评阅或客观题分数最高，均取总分最高
        if (MarkingType.ALL.toString().equals(markingType) || MarkingType.OBJECT_SCORE_MAX.toString().equals(markingType)) {
            examScoreEntity = curExamScoreList.stream().max(Comparator.comparing(p -> p.getTotalScore())).get();
        } else if (MarkingType.LAST_SUBMIT.toString().equals(markingType)) {
            examScoreEntity = curExamScoreList.stream().max(Comparator.comparing(p -> p.getId())).get();
        } else {
            throw new StatusException("201001", "阅卷类型不支持");
        }
        return examScoreEntity;
    }

}
